/* -*-  Mode:C; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1999, 2000 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the Linux source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CMAUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CMAUTHORS file.
 */
/*
 * An implementation of the Congestion Manager (CM) for Linux.
 * Binomial congestion control algorithms.
 */
#include <net/cm.h>

#ifndef _CM_BINCONGCTL_H
#define _CM_BINCONGCTL_H

#define CM_RTT_DEFAULT 1000	/* in milliseconds */
#define CM_BINCC_MAXBURST 4

extern void cmbincc_init(cm_congctl_t *cmcc);
extern void cmbincc_update(cm_congctl_t *cmbincc, int nsent, int nrecd, 
			   __u32 rtt, int lossmode);
extern void cmbincc_query(cm_congctl_t *cmbincc, double *bw, double *srtt, 
		       double *rttvar);
extern void cmbincc_notify(cm_congctl_t *cmbincc, int nsent);
extern void cmbincc_startup(cm_congctl_t *cmbincc, int nrecd, double rtt,
			    int lossmode);
void cmbincc_restart(cm_congctl_t *cmbincc, u32 now);
extern inline void cmbincc_decrease(cm_congctl_t *cmbincc, u32 now);
extern inline void cmbincc_increase(cm_congctl_t *cmbincc, int nrecd);
extern inline void cmbincc_update_rtt(cm_congctl_t *cmbincc, u32 rtt);
extern void cmbincc_startup(cm_congctl_t *cmbincc, int nrecd, double rtt, 
			    int lossmode);
extern void cmbincc_wakeup(cm_congctl_t *cmbincc);

#endif
